@@ -5,6 +5,7 @@
 */
 
 #include "nsHebrewProber.h"
+#include "nsSBCharSetProber.h"
 #include <stdio.h>
 
 // windows-1255 / ISO-8859-8 code points of interest
@@ -30,14 +31,14 @@
 #define VISUAL_HEBREW_NAME ("ISO-8859-8")
 #define LOGICAL_HEBREW_NAME ("windows-1255")
 
-namespace kencodingprober
+namespace
 {
-bool nsHebrewProber::isFinal(char c)
+bool isFinal(char c)
 {
     return ((c == FINAL_KAF) || (c == FINAL_MEM) || (c == FINAL_NUN) || (c == FINAL_PE) || (c == FINAL_TSADI));
 }
 
-bool nsHebrewProber::isNonFinal(char c)
+bool isNonFinal(char c)
 {
     return ((c == NORMAL_KAF) || (c == NORMAL_MEM) || (c == NORMAL_NUN) || (c == NORMAL_PE));
     // The normal Tsadi is not a good Non-Final letter due to words like
@@ -51,7 +52,15 @@ bool nsHebrewProber::isNonFinal(char c)
     // these letters as Non-Final letters outweighs the damage since these words
     // are quite rare.
 }
+} // namespace <anonymous>
 
+namespace kencodingprober
+{
+nsHebrewProber::nsHebrewProber()
+    : mLogicalProb(new nsSingleByteCharSetProber<false>(&Win1255Model))
+    , mVisualProb(new nsSingleByteCharSetProber<true>(&Win1255Model))
+{
+}
 /** HandleData
  * Final letter analysis for logical-visual decision.
  * Look for evidence that the received buffer is either logical Hebrew or
@@ -79,6 +88,9 @@ bool nsHebrewProber::isNonFinal(char c)
  */
 nsProbingState nsHebrewProber::HandleData(const char *aBuf, unsigned int aLen)
 {
+    mLogicalProb->HandleData(aBuf, aLen);
+    mVisualProb->HandleData(aBuf, aLen);
+
     // Both model probers say it's not them. No reason to continue.
     if (GetState() == eNotMe) {
         return eNotMe;
@@ -86,10 +98,9 @@ nsProbingState nsHebrewProber::HandleData(const char *aBuf, unsigned int aLen)
 
     const char *curPtr;
     const char *endPtr = aBuf + aLen;
-    char cur;
 
-    for (curPtr = (char *)aBuf; curPtr < endPtr; ++curPtr) {
-        cur = *curPtr;
+    for (curPtr = aBuf; curPtr < endPtr; ++curPtr) {
+        char cur = *curPtr;
         if (cur == ' ') { // We stand on a space - a word just ended
             if (mBeforePrev != ' ') { // *(curPtr-2) was not a space so prev is not a 1 letter word
                 if (isFinal(mPrev)) { // case (1) [-2:not space][-1:final letter][cur:space]
@@ -111,6 +122,25 @@ nsProbingState nsHebrewProber::HandleData(const char *aBuf, unsigned int aLen)
     return eDetecting;
 }
 
+float nsHebrewProber::GetConfidence()
+{
+    if (GetState() == eNotMe) {
+        return 0.01f;
+    }
+
+    int finalsub = mFinalCharLogicalScore - mFinalCharVisualScore;
+    auto logicalConfidence = mLogicalProb->GetConfidence();
+    auto visualConfidence = mVisualProb->GetConfidence();
+
+    if ((logicalConfidence - 0.1 > visualConfidence) && (finalsub >= 0)) {
+        return logicalConfidence;
+    } else if ((visualConfidence - 0.1 > logicalConfidence) && (finalsub <= 0)) {
+        return visualConfidence;
+    } else {
+        return 0.01f;
+    }
+}
+
 // Make the decision: is it Logical or Visual?
 const char *nsHebrewProber::GetCharSetName()
 {
@@ -164,7 +194,9 @@ nsProbingState nsHebrewProber::GetState(void)
 #ifdef DEBUG_PROBE
 void nsHebrewProber::DumpStatus()
 {
-    printf("  HEB: %d - %d [Logical-Visual score]\r\n", mFinalCharLogicalScore, mFinalCharVisualScore);
+    printf("  HEB: [%.3f] %d - %d [Logical-Visual score]:\r\n", GetConfidence(), mFinalCharLogicalScore, mFinalCharVisualScore);
+    mLogicalProb->DumpStatus();
+    mVisualProb->DumpStatus();
 }
 #endif
 }
